/*
 * Copyright (c) 2010, Gregory W Hewitt. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Gregory W Hewitt or the names of
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/**
 * The Round Robin Starting Heat Line Up Calculator (RRobinCalc6).
 * This program is a simple round robin calculator for multiple lane races and
 * multiple races per lane.  For example a track may have 4 lanes and may race
 * two times in each lane.  This program will calculate the optimum number of
 * participants, and the starting lineup so in each lane, the car number will
 * be incremented by 1 with each successive heat, and each racer will race
 * every other racer.
 *
 * This version of the calculater is different from the others in that the
 * previous version took a brute force approach and incremented to the next
 * racer regardless if it was already used.  They then checked if it was a
 * valid linup.  One of the previous versions checked if it could skip a bunch,
 * but still took quite a bit of time to increment through all the positions.
 *
 * This version takes a much more mathematical or logical approach.
 * A position will select the next racer based on what is available
 * (or handed to it).  It then will check if it is a valid racer for that
 * position and what other racers get taken also, based on incrementing through
 * the lanes.  If it is valid, it creates another list of available racers minus
 * what was used and passes it on to the next position.  If no valid racer is
 * found for a position, it reports back to the previous position that it failed
 * and the previous position will increment and provide an updated list of
 * available racers.  This is more of a recursive algorithm and is the fastest
 * of the different algorithms I have tried.
 *
 * Improvements to algorithm from 5 to 5B include reduced array indexing (only
 * one dimention array vice two).  Also use inheritance and override to
 * eliminate a couple of if statements.  Not much improvement in speed but it
 * is a little.
 *
 * From 5B to 6, implemented additional functions.
 * First, implemented calculate all possible combinations for given parameters.
 * With this function selected, if no combinations are found, the program will
 * halt and report the fact that none were found.  It will not increment the
 * number of racers as when it is searching for the first possibility.  Also,
 * the B1 shortcut is not available.
 * Second, implemented calculation for lineups with racers not in consecutive
 * heats, but still an incremental round robin line up.  This results in a non
 * perfect round robin line up with at least 2 more racers required.  Each racer
 * will not race against the number before them nor after them.
 *
 * From 6 to 6_1, implemented calculate entire lineup
 * 
 * @author ghewitt
 */

package rrobincalc6;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;

public class RRobinCalcMain extends JFrame {
    // panel fields
    private JTabbedPane tabbedPane;
    private JPanel calcPanel;
    private JTextArea outputTextArea;
    private JScrollPane outputScrollPane;
    private JTextField numberOfLanesTextField;
    private JTextField racesPerLaneTextField;
    private JTextField idealNumOfRacersTextField;
    private JTextField addRacersTextField;
    private JTextField numberOfRacersTextField;
    private JTable resultsTable;
    private JButton calculateButton;
    private JButton stopButton;
    private ResultsTableModel resultsTableModel;
    private JScrollPane resultsScrollPane;
    private JCheckBox notConsecutiveCheckBox;
    private JRadioButton normalCalcRadioButton;
    private JRadioButton findAllRadioButton;
    private JRadioButton findAllFilterRadioButton;
    private JRadioButton useShortCutRadioButton;
    private JRadioButton entireLnUpRadioButton;
    private JRadioButton entireLnUpShortRadioButton;
    private JRadioButton displayAllRadioButton;
    private JRadioButton displayRowOneRadioButton;
    private JRadioButton displayThreeRadioButton;
    private JRadioButton displayNoneRadioButton;
    private ButtonGroup displayButtonGroup;
    private ButtonGroup calcButtonGroup;
    private int numOfRacers;
    private int numOfLanes;
    private int racesPerLane;
    private int addRacers;
    private int startLineUp[][];
    private boolean validLineUp;
    private Thread calculateRRThread;
    private CalculateRoundRobin calcRR;

    public RRobinCalcMain() {
        super("Round Robin Calculator");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setLayout(new GridLayout(1,1));

        calcPanel = new JPanel();
        calcPanel.setLayout(new GridBagLayout());
        setMinimumSize(new Dimension(375, 538));
        GridBagConstraints c = new GridBagConstraints();

        outputTextArea = new JTextArea(5,24);
        outputScrollPane = new JScrollPane(outputTextArea);
        outputTextArea.setEditable(false);
        outputTextArea.setFont(new Font("Monospaced", Font.PLAIN, 12));

        validLineUp = false;
        numOfRacers = 0;
        numOfLanes = 4;
        racesPerLane = 1;
        addRacers = 0;

        c.gridx = 0;
        c.gridy = 0;
        c.insets = new Insets(2,5,3,5);
        c.anchor = GridBagConstraints.LINE_START;
        c.gridwidth = 2;
        calcPanel.add(new JLabel("Number of Lanes:"), c);
        c.gridy = 1;
        calcPanel.add(new JLabel("Races Per Lane:"), c);
        c.gridy = 2;
        calcPanel.add(new JLabel("Ideal Number of Racers:"), c);
        c.gridy = 3;
        calcPanel.add(new JLabel("Additional Racers Above Ideal:"), c);
        c.gridwidth = 1;

        numberOfLanesTextField = new JTextField(3);
        numberOfLanesTextField.setText("" + numOfLanes);
        numberOfLanesTextField.addFocusListener(new LanesFocusListener());
        c.gridx = 2;
        c.gridy = 0;
        calcPanel.add(numberOfLanesTextField, c);

        racesPerLaneTextField = new JTextField(3);
        racesPerLaneTextField.setText("" + racesPerLane);
        racesPerLaneTextField.addFocusListener(new RacesFocusListener());
        c.gridy = 1;
        calcPanel.add(racesPerLaneTextField, c);

        idealNumOfRacersTextField = new JTextField(3);
        idealNumOfRacersTextField.setEditable(false);
        idealNumOfRacersTextField.setFocusable(false);
        idealNumOfRacersTextField.setText("13");
        c.gridy = 2;
        calcPanel.add(idealNumOfRacersTextField, c);

        addRacersTextField = new JTextField(3);
        addRacersTextField.setText("" + addRacers);
        addRacersTextField.addFocusListener(new AddRacersFocusListener());
        c.gridy = 3;
        calcPanel.add(addRacersTextField, c);

        notConsecutiveCheckBox = new JCheckBox("Nonconsecutive Lineup");
        c.gridwidth = 3;
        c.gridx = 0;
        c.gridy = 4;
        calcPanel.add(notConsecutiveCheckBox,c);

        normalCalcRadioButton = new JRadioButton("Normal Calculations");
        normalCalcRadioButton.setSelected(true);
        c.gridy = 5;
        calcPanel.add(normalCalcRadioButton,c);

        useShortCutRadioButton = new JRadioButton("Use B1 Short Cut");
        c.gridy = 6;
        calcPanel.add(useShortCutRadioButton,c);

        entireLnUpRadioButton = new JRadioButton("Entire Lineup Normal Calc");
        c.gridy = 7;
        calcPanel.add(entireLnUpRadioButton,c);

        entireLnUpShortRadioButton = new JRadioButton("Entire Lineup Short Cut");
        c.gridy = 8;
        calcPanel.add(entireLnUpShortRadioButton, c);

        findAllRadioButton = new JRadioButton("Find All Possible");
        c.gridy = 9;
        calcPanel.add(findAllRadioButton,c);

        findAllFilterRadioButton = new JRadioButton("Find All Possible Filtered");
        c.gridy = 10;
        calcPanel.add(findAllFilterRadioButton,c);

        calcButtonGroup = new ButtonGroup();
        calcButtonGroup.add(normalCalcRadioButton);
        calcButtonGroup.add(findAllRadioButton);
        calcButtonGroup.add(findAllFilterRadioButton);
        calcButtonGroup.add(useShortCutRadioButton);
        calcButtonGroup.add(entireLnUpRadioButton);
        calcButtonGroup.add(entireLnUpShortRadioButton);

        calculateButton = new JButton("Calculate");
        calculateButton.addActionListener(new CalculateActionListener());
        c.gridx = 0;
        c.gridy = 11;
        c.gridwidth = 1;
        calcPanel.add(calculateButton, c);

        resultsTableModel = new ResultsTableModel();
        resultsTable = new JTable(resultsTableModel);
        resultsScrollPane = new JScrollPane(resultsTable);
        resultsScrollPane.setPreferredSize(new Dimension(350, 100));
        resultsScrollPane.setBorder(BorderFactory.createTitledBorder("Calculation Display:"));
        resultsTable.setEnabled(false);
        resultsTable.setFocusable(false);
        c.gridx = 0;
        c.gridy = 12;
        c.gridwidth = 4;
        c.fill = GridBagConstraints.BOTH;
        c.weighty = 1;
        calcPanel.add(resultsScrollPane, c);
        c.fill = GridBagConstraints.NONE;
        c.weighty = 0;

        c.gridy = 13;
        c.gridwidth = 2;
        calcPanel.add(new JLabel("Number of Racers Needed:"), c);

        numberOfRacersTextField = new JTextField(3);
        numberOfRacersTextField.setEditable(false);
        numberOfRacersTextField.setFocusable(false);
        c.gridx = 2;
        c.gridwidth = 1;
        calcPanel.add(numberOfRacersTextField, c);

        c.gridx = 1;
        c.gridy = 11;
        stopButton = new JButton("Stop");
        stopButton.addActionListener(new stopListener());
        stopButton.setEnabled(false);
        calcPanel.add(stopButton, c);

        c.gridx = 3;
        c.gridy = 0;
        c.weightx = 1;
        calcPanel.add(new JLabel("Display Calculations"), c);
        c.gridy = 1;
        displayAllRadioButton = new JRadioButton("All");
        displayAllRadioButton.setFocusable(false);
        calcPanel.add(displayAllRadioButton, c);

        c.gridy = 2;
        displayRowOneRadioButton = new JRadioButton("First Row ");
        displayRowOneRadioButton.setSelected(true);
        displayRowOneRadioButton.setFocusable(false);
        calcPanel.add(displayRowOneRadioButton, c);

        c.gridy = 3;
        displayThreeRadioButton = new JRadioButton("First Three cells");
        displayThreeRadioButton.setFocusable(false);
        calcPanel.add(displayThreeRadioButton, c);

        c.gridy = 4;
        displayNoneRadioButton = new JRadioButton("None");
        displayNoneRadioButton.setFocusable(false);
        calcPanel.add(displayNoneRadioButton, c);

        displayButtonGroup = new ButtonGroup();
        displayButtonGroup.add(displayAllRadioButton);
        displayButtonGroup.add(displayRowOneRadioButton);
        displayButtonGroup.add(displayThreeRadioButton);
        displayButtonGroup.add(displayNoneRadioButton);

        tabbedPane = new JTabbedPane();
        tabbedPane.addTab("Calc", calcPanel);
        tabbedPane.addTab("Output", outputScrollPane);

        add(tabbedPane);
        pack();
        setVisible(true);


        System.out.println(this.getSize());
        outputTextArea.setText("");
    }


    private class CalculateRoundRobin implements Runnable  {
        private ArrayList<lineUpEntry> lineUpArray;
        private boolean running;
        private boolean racerTest[];
        private int maxTextRows;
        @Override
        public void run()  {
            // Actual thread for calculating the round robin heat line ups
            validLineUp = false;
            running = true;
            maxTextRows = 4000000 * (racesPerLane+2)/(5*numOfLanes*racesPerLane+12);
            lineUpArray = new ArrayList<lineUpEntry>();
            lineUpArray.clear();

            numOfRacers = numOfLanes * (numOfLanes-1) * racesPerLane + 1 + addRacers;
            numberOfRacersTextField.setText(""+ numOfRacers);
            numberOfRacersTextField.setBackground(getBackground());

            startLineUp = new int[racesPerLane][numOfLanes];
            outputTextArea.setText("");

            Thread.yield();

            // setup positions and initialize;
            Position[][] incPositions =new Position[racesPerLane][numOfLanes-1];
            Position tempPos = null;
            for(int row = racesPerLane-1; row >= 0; row--)  {
                for(int col = numOfLanes-2; col >= 0; col--)  {
                    if(tempPos == null)  {
                        if(findAllRadioButton.isSelected() || findAllFilterRadioButton.isSelected())  {
                            incPositions[row][col] = new EndAllPosition(row, col+1, tempPos);
                        }
                        else  {
                            incPositions[row][col] = new EndPosition(row, col+1, tempPos);
                        }
                    }
                    else  {
                        if(displayAllRadioButton.isSelected())  {
                            incPositions[row][col] = new DisplayPosition(row, col+1, tempPos);
                        }
                        else if(displayRowOneRadioButton.isSelected() && row == 0)  {
                            incPositions[row][col] = new DisplayPosition(row, col+1, tempPos);
                        }
                        else if(displayThreeRadioButton.isSelected() && row == 0 && col < 2)  {
                            incPositions[row][col] = new DisplayPosition(row, col+1, tempPos);
                        }
                        else
                            incPositions[row][col] = new Position(row, col+1, tempPos);
                    }
                    tempPos = incPositions[row][col];
                }
            }

            if(findAllRadioButton.isSelected() || findAllFilterRadioButton.isSelected())  {
                tabbedPane.setSelectedIndex(1);
            }

            // Loop untill startLineUp is valid
            while(!validLineUp && running)  {
                validLineUp = true;

                // initialize starting lineup
                for (int row = 0; row < racesPerLane; row++)
                    for (int col = 0; col < numOfLanes; col++)
                        startLineUp[row][col] = 0;
                resultsTableModel.setData(startLineUp);

                // create and clears test array
                racerTest = new boolean[numOfRacers];
                racerTest[0] = true;
                 for (int row =1; row < numOfRacers; row ++)  {
                     racerTest[row] = false;
                 }

                // set increment position array sizes
                for(int row = 0; row < incPositions.length; row++)  {
                    for(int col = 0; col < incPositions[0].length; col++)  {
                        incPositions[row][col].setMaxRacers(numOfRacers);
                    }
                }

                // use short cut
                if(useShortCutRadioButton.isSelected() ||
                        entireLnUpShortRadioButton.isSelected())  {
                    // create initial available lineup
                    if (notConsecutiveCheckBox.isSelected())  {
                        startLineUp[0][1] = 2;
                        racerTest[1] = true;
                        racerTest[numOfRacers-1] = true;
                        racerTest[2] = true;
                        racerTest[numOfRacers-2] = true;
                        resultsTableModel.setValueAt(2, 0, 1);
                        for (int i=3; i<numOfRacers-2; i++)  {
                            incPositions[0][1].addRacer(i);
                        }
                        
                    }
                    else {
                        startLineUp[0][1] = 1;
                        racerTest[1] = true;
                        racerTest[numOfRacers-1] = true;
                        resultsTableModel.setValueAt(1, 0, 1);
                        for (int i=2; i<numOfRacers-1; i++)  {
                            incPositions[0][1].addRacer(i);
                        }
                    }
                    // call first postion to begin calculations
                    validLineUp = incPositions[0][1].calcPosition();
                }
                else  {
                    if (notConsecutiveCheckBox.isSelected())  {
                        racerTest[1] = true;
                        racerTest[numOfRacers-1] = true;
                        // create initial available lineup
                        for (int i=2; i< numOfRacers-1; i++)  {
                            incPositions[0][0].addRacer(i);
                        }
                    }
                    else  {
                        // create initial available lineup
                        for (int i=1; i<numOfRacers; i++)  {
                            incPositions[0][0].addRacer(i);
                        }
                    }
                    // call first postion to begin calculations
                    validLineUp = incPositions[0][0].calcPosition();
                }

                // if not a valid lineup, increments and resize of arrays
                if (!validLineUp && running)  {
                    if (findAllRadioButton.isSelected()||findAllFilterRadioButton.isSelected())  {
                        running = false;
                        lineUpArray.clear();
                    }
                    else  {
                        numOfRacers ++;
                        numberOfRacersTextField.setText(""+ numOfRacers);
                        numberOfRacersTextField.setBackground(Color.YELLOW);
                    }
                }
            }

            // display results
            resultsTableModel.setData(startLineUp);
            if ((entireLnUpRadioButton.isSelected() ||
                    entireLnUpShortRadioButton.isSelected())&& validLineUp)  {
                tabbedPane.setSelectedIndex(1);
                outputTextArea.append("\n********************\n");
                outputTextArea.append("** Entire Lineup  **\n");
                outputTextArea.append("********************\n");
                String tempS = "";
                int tempI = 0;
                for (int stage = 0; stage < racesPerLane; stage++)  {
                    outputTextArea.append("\nStage: " + (stage + 1)+ "\n");
                    outputTextArea.append("--------------------\n");

                    for (int heat = 0; heat < numOfRacers; heat++)  {
                        tempS = "";
                        for (int lane = 0; lane< numOfLanes; lane++)  {
                            tempI = (heat+startLineUp[stage][lane])%numOfRacers;
                            tempS = tempS + String.format("%4d",tempI) +",";
                        }
                        outputTextArea.append(tempS+"\n");
                    }
                }

            }
            // setup for next input
            calculateButton.setEnabled(true);
            numberOfLanesTextField.setEnabled(true);
            racesPerLaneTextField.setEnabled(true);
            addRacersTextField.setEnabled(true);
            findAllRadioButton.setEnabled(true);
            findAllFilterRadioButton.setEnabled(true);
            notConsecutiveCheckBox.setEnabled(true);
            normalCalcRadioButton.setEnabled(true);
            entireLnUpRadioButton.setEnabled(true);
            displayAllRadioButton.setEnabled(true);
            displayRowOneRadioButton.setEnabled(true);
            displayThreeRadioButton.setEnabled(true);
            displayNoneRadioButton.setEnabled(true);
            stopButton.setEnabled(false);
            if(numOfLanes > 2){
                useShortCutRadioButton.setEnabled(true);
                entireLnUpShortRadioButton.setEnabled(true);
            }
        }

        public void stopCalc()  {
            running = false;
        }

        // Internal private class for each possition.
        // positions will only check current row up to themselves, no need to
        // go beyond or check other rows.

        private class Position  {
            protected Position nextPosition;
            protected int positionRow, positionCol, racerCount;
//            protected int[] startLineUpRow;
            protected int[] availableRacers;

            public Position(int row, int col, Position next)  {
                nextPosition = next;
                positionRow = row;
                positionCol = col;
//                startLineUpRow = startLineUp[row];
                availableRacers = null;
                racerCount = 0;
            }

            public void setMaxRacers(int mracer)  {
                availableRacers = new int[mracer];
                racerCount = 0;
            }

            public void addRacer(int racer)  {
                availableRacers[racerCount] = racer;
                racerCount++;
            }

            public boolean calcPosition()  {
                boolean validPosition = false;
                boolean rTest[] = racerTest;
                int index=0, tempR;
                int currentRacer, checkRacer, tNOR = numOfRacers;
                int rCount = racerCount, pCol = positionCol;
                int startLineUpRow[] = startLineUp[positionRow];
                int aRacers[] = availableRacers;

                while(!validPosition && index < racerCount && running)  {
                    // select racer number;
                    validPosition = true;
                    currentRacer = aRacers[index];

                    // can't use the following shortcut, if num is its own
                    // complement, don't perform check.  Ouch
//                    rTest[currentRacer] = true;
//                    rTest[tNOR - currentRacer] = true;

                    // check if valid
                    for(int col=0; col<pCol && validPosition; col++)  {
                        // check if positionCol is zero column
                        tempR = startLineUpRow[col];
                        checkRacer = currentRacer - tempR ;
                        if(checkRacer <0) { checkRacer += tNOR; }
                        if(rTest[checkRacer])  {
                            validPosition = false;
                        }
                        else   {
                            rTest[checkRacer] = true;

                            // check if col is zero column
    //                        checkRacer = currentRacer - tempR;
    //                        if(checkRacer <0) { checkRacer += tNOR; }
                            checkRacer = tNOR - checkRacer;
                            if(rTest[checkRacer])  {
                                validPosition = false;
                            }
                            else {
                                rTest[checkRacer] = true;
                            }
                        }
                    }

                    // if valid create available lineup and call next position
                    if(validPosition)  {
                        startLineUpRow[pCol] = currentRacer;
                        for (int i = 0; i < rCount; i++) {
                            if (!rTest[aRacers[i]]) {
                                nextPosition.addRacer(aRacers[i]);
                            }
                        }
                        validPosition = nextPosition.calcPosition();
                    }
                    if(!validPosition)  { // if not valid, clear checks, increment index;
                        index++;
                        for (int i=0; i<rCount; i++)  {
                            rTest[aRacers[i]] = false;
                        }
                    }
                }
                racerCount = 0;
                return validPosition;
            }
        }
        private class EndPosition extends Position  {
            public EndPosition (int row, int col, Position next)  {
                super(row, col, next);
            }
            @Override
            public boolean calcPosition()  {
                boolean validPosition = false;
                boolean rTest[] = racerTest;
                int index=0, tempR;
                int currentRacer, checkRacer, tNOR = numOfRacers;
                int rCount = racerCount, pCol = positionCol;
                int startLineUpRow[] = startLineUp[positionRow];
                int aRacers[] = availableRacers;

                while(!validPosition && index < racerCount && running)  {
                    // select racer number;
                    validPosition = true;
                    currentRacer = aRacers[index];

//                    rTest[currentRacer] = true;
//                    rTest[tNOR - currentRacer] = true;

                    // check if valid
                    for(int col=0; col<pCol && validPosition; col++)  {
                        // check if positionCol is zero column
                        tempR = startLineUpRow[col];
                        checkRacer = currentRacer - tempR;
                        if(checkRacer <0) { checkRacer += tNOR; }
                        if(rTest[checkRacer])  {
                            validPosition = false;
                        }
                        else  {
                            rTest[checkRacer] = true;

                            // check if col is zero column
    //                        checkRacer = currentRacer - tempR;
    //                        if(checkRacer <0) { checkRacer += tNOR; }
                            checkRacer = tNOR - checkRacer;
                            if(rTest[checkRacer])  {
                                validPosition = false;
                            }
                            else  {
                                rTest[checkRacer] = true;
                            }
                        }
                    }
                    // if valid output results to test area
                    if(validPosition)  {
                        String temp = "";
                        startLineUpRow[pCol] = currentRacer;
                        outputTextArea.append("Number of racers: " + tNOR + "\n");
                        outputTextArea.append("Starting Heat Lineups:\n");
                        for (int i=0; i< startLineUp.length; i++)  {
                            for (int j=0; j<startLineUp[0].length; j++)  {
                                temp = temp+ String.format("%3d, ", startLineUp[i][j]);
                            }
                            outputTextArea.append(temp + "\n");
                            temp = "";
                        }
                        outputTextArea.append("\n");
                    }
                    if(!validPosition)  { // if not valid, clear checks, increment index;
                        index++;
                        for (int i=0; i<rCount; i++)  {
                            rTest[aRacers[i]] = false;
                        }
                    }
                }
                racerCount = 0;
                return validPosition;
            }

        }
        private class EndAllPosition extends Position  {
            int count;
            public EndAllPosition (int row, int col, Position next)  {
                super(row, col, next);
                count = 0;
            }
            @Override
            public boolean calcPosition()  {
                boolean validPosition = false;
                boolean rTest[] = racerTest;
                int index=0, tempR;
                int currentRacer, checkRacer, tNOR = numOfRacers;
                int rCount = racerCount, pCol = positionCol;
                int startLineUpRow[] = startLineUp[positionRow];
                int aRacers[] = availableRacers;

                while(!validPosition && index < racerCount && running)  {
                    // select racer number;
                    validPosition = true;
                    currentRacer = aRacers[index];

//                    rTest[currentRacer] = true;
//                    rTest[tNOR - currentRacer] = true;

                    // check if valid
                    for(int col=0; col<pCol && validPosition; col++)  {
                        // check if positionCol is zero column
                        tempR = startLineUpRow[col];
                        checkRacer = currentRacer - tempR;
                        if(checkRacer <0) { checkRacer += tNOR; }
                        if(rTest[checkRacer])  {
                            validPosition = false;
                        }
                        else  {
                            rTest[checkRacer] = true;

                            // check if col is zero column
    //                        checkRacer = currentRacer - tempR;
    //                        if(checkRacer <0) { checkRacer += tNOR; }
                            checkRacer = tNOR - checkRacer;
                            if(rTest[checkRacer])  {
                                validPosition = false;
                            }
                            else  {
                                rTest[checkRacer] = true;
                            }
                        }
                    }
                    // if valid, add to output text area and then continue on
                    if(validPosition)  {
                        startLineUpRow[pCol] = currentRacer;
                        if(findAllFilterRadioButton.isSelected())  {

                            // filter out of order sets in each row
                            for (int i=0; i<startLineUp.length; i++)  {
                                for (int j=0; j<startLineUp[0].length-1; j++)  {
                                    for (int k=j+1; k<startLineUp[0].length; k++){
                                        if(startLineUp[i][k] < startLineUp[i][j])  {
                                            validPosition = false;
                                        }
                                    }
                                }
                            }

                            // filter out of order rows, using column 1 only
                            for (int i=0; i< startLineUp.length-1; i++)  {
                                for (int j=i+1; j<startLineUp.length; j++)  {
                                    if (startLineUp[j][1] < startLineUp[i][1])  {
                                        validPosition = false;
                                    }
                                }
                            }

                            // filter rotated row sets
                            boolean match;
                            // increment through entire array
                            for(int i=0; i< lineUpArray.size() && validPosition; i++)  {
                                // increment through rows
                                match = true;
                                for (int j=0; j< startLineUp.length; j++) {
                                    if (!lineUpArray.get(i).isEqualTo(startLineUp[j]))  {
                                        match = false;
                                    }
                                }
                                if (match)  {
                                    validPosition = false;
                                }
                            }

                            // Add current entry to lineUpArray
                            if(validPosition)  {
                                lineUpArray.add(new lineUpEntry(startLineUp));
                            }
                        }
                    }
                    if(validPosition)  {
                        String temp = "";
                        if(outputTextArea.getLineCount() < maxTextRows)  {
                            outputTextArea.append(tNOR + ": " +count+ "\n");
                            for (int i=0; i< startLineUp.length; i++)  {
                                for (int j=0; j<startLineUp[0].length; j++)  {
                                    temp = temp+ String.format("%3d, ", startLineUp[i][j]);
                                }
                                outputTextArea.append(temp + "\n");
                                temp = "";
                            }
                            outputTextArea.append("\n");
                            validPosition = false;
                            count ++;
                        }
                        else {
                            running = false;
                            outputTextArea.append("Maximum output text rows reached.");
                        }
                    }
                    if(!validPosition)  { // if not valid, clear checks, increment index;
                        index++;
                        for (int i=0; i<rCount; i++)  {
                            rTest[aRacers[i]] = false;
                        }
                    }
                }
                racerCount = 0;
                return validPosition;
            }

        }
        private class DisplayPosition extends Position  {
            public DisplayPosition (int row, int col, Position next)  {
                super(row, col, next);
            }
            @Override
            public boolean calcPosition()  {
                boolean validPosition = false;
                boolean rTest[] = racerTest;
                int index=0, tempR;
                int currentRacer, checkRacer, tNOR = numOfRacers;
                int rCount = racerCount, pCol = positionCol;
                int startLineUpRow[] = startLineUp[positionRow];
                int aRacers[] = availableRacers;

                while(!validPosition && index < racerCount && running)  {
                    // select racer number;
                    validPosition = true;
                    currentRacer = aRacers[index];

//                    rTest[currentRacer] = true;
//                    rTest[tNOR - currentRacer] = true;

                    // check if valid
                    for(int col=0; col<pCol && validPosition; col++)  {
                        // check if positionCol is zero column
                        tempR = startLineUpRow[col];
                        checkRacer = currentRacer - tempR;
                        if(checkRacer <0) { checkRacer += tNOR; }
                        if(rTest[checkRacer])  {
                            validPosition = false;
                        }
                        else  {
                            rTest[checkRacer] = true;

                            // check if col is zero column
    //                        checkRacer = currentRacer - tempR;
    //                        if(checkRacer <0) { checkRacer += tNOR; }
                            checkRacer = tNOR - checkRacer;
                            if(rTest[checkRacer])  {
                                validPosition = false;
                            }
                            else  {
                                rTest[checkRacer] = true;
                            }
                        }
                    }
                    // if valid create available lineup and call next position
                    if(validPosition)  {
                        startLineUpRow[pCol] = currentRacer;
                        resultsTableModel.setValueAt(currentRacer, positionRow, pCol);
                        for (int i = 0; i < rCount; i++) {
                            if (!rTest[aRacers[i]]) {
                                nextPosition.addRacer(aRacers[i]);
                            }
                        }
                        validPosition = nextPosition.calcPosition();
                    }
                    if(!validPosition)  { // if not valid, clear checks, increment index;
                        index++;
                        for (int i=0; i<rCount; i++)  {
                            rTest[aRacers[i]] = false;
                        }
                    }
                }
                racerCount = 0;
                return validPosition;
            }

        }

        private class lineUpEntry  {
            public int[][] lineUp;

            public lineUpEntry(int[][] entry)  {
                lineUp = new int[entry.length][entry[0].length];
                for(int i=0; i< entry.length; i++)  {
                    for (int j=0; j<entry[0].length; j++)  {
                        lineUp[i][j] = entry[i][j];
                    }
                }
            }
            public boolean isEqualTo(int line[])  {
                boolean match;
                int [] tempLine = new int[line.length];
                int t;

                // copy
                for (int i=0; i< line.length; i++)  {
                    tempLine[i] = line[i];
                }

                // compair
                match = false;
                for (int row = 0; row < lineUp.length && !match; row++) { // search all rows
                    for (int i=0; i< tempLine.length && !match; i++)  {  // increment rotation
                        // compair
                        match = true;
                        for(int j=0; j< tempLine.length; j++)  {
                            if(tempLine[j] != lineUp[row][j])
                                match = false;
                        }

                        if(!match)  { // if no match, rotate tempLine and zero out
                            t = tempLine[0];
                            for (int k = 1; k < tempLine.length; k++)  {
                                tempLine[k-1] = tempLine[k];
                            }
                            tempLine[tempLine.length-1] = t;

                            // zero out tempLine
                            t = tempLine[0];
                            for(int k = 0; k < tempLine.length; k++)  {
                                tempLine[k] = tempLine[k] - t;
                                if(tempLine[k] <0)   {
                                    tempLine[k] += numOfRacers;
                                }
                            }
                        }
                    }
                }

                return match;
            }
        }
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        new RRobinCalcMain();

    }

    private class ResultsTableModel extends AbstractTableModel  {
        private Integer data[][];
//        private Date oDate;
//        private Date nDate;

        public ResultsTableModel() {
            data = new Integer[1][4];
//            oDate = new Date();
//            nDate = null;

            for(int row=0; row<1; row++)
                for (int col=0; col<4; col++)
                    data[row][col] = new Integer(0);

        }

        synchronized public void setData(int[][] newData)  {
            if(newData[0].length != data[0].length)  {
                data = new Integer[newData.length][newData[0].length];
                fireTableStructureChanged();
            }
            if(newData.length != data.length)
                data = new Integer[newData.length][newData[0].length];

            for(int row=0; row< newData.length; row++)
                for(int col=0; col<newData[0].length; col++)
                    data[row][col] = new Integer(newData[row][col]);

            fireTableDataChanged();
        }

        @Override
        synchronized public int getRowCount() {
            return data.length;
        }

        @Override
        synchronized public int getColumnCount() {
            return data[0].length;
        }

        @Override
        synchronized public Object getValueAt(int rowIndex, int columnIndex) {
            return data[rowIndex][columnIndex];
        }
        synchronized public void setValueAt(Integer value, int row, int col)  {
            data[row][col] = value;
//            if (row == 0 && col == 2)  {
//                nDate = new Date();
//                System.out.println(nDate.getTime() - oDate.getTime());
//                oDate = nDate;
//            }

            fireTableCellUpdated(row, col);
        }
    }

    private class CalculateActionListener implements ActionListener  {

        @Override
        public void actionPerformed(ActionEvent e) {

            // disables everything untill finished
            numberOfRacersTextField.setBackground(getBackground());
            calculateButton.setEnabled(false);
            numberOfLanesTextField.setEnabled(false);
            racesPerLaneTextField.setEnabled(false);
            findAllRadioButton.setEnabled(false);
            findAllFilterRadioButton.setEnabled(false);
            notConsecutiveCheckBox.setEnabled(false);
            normalCalcRadioButton.setEnabled(false);
            useShortCutRadioButton.setEnabled(false);
            entireLnUpRadioButton.setEnabled(false);
            entireLnUpShortRadioButton.setEnabled(false);
            addRacersTextField.setEnabled(false);
            displayAllRadioButton.setEnabled(false);
            displayRowOneRadioButton.setEnabled(false);
            displayThreeRadioButton.setEnabled(false);
            displayNoneRadioButton.setEnabled(false);
            stopButton.setEnabled(true);
            stopButton.setBackground(calculateButton.getBackground());

            // verifies the inputs and starts calculations.
            boolean validInput = true;
            int lanes = 0, rpl = 0, addr = 0;
            try  {
                lanes = Integer.parseInt(numberOfLanesTextField.getText());
                if(lanes < 2) throw new Exception();
            }
            catch(Exception err)  {
                validInput = false;
                numberOfLanesTextField.setText("" + numOfLanes);
            }

            try  {
                rpl = Integer.parseInt(racesPerLaneTextField.getText());
                if(rpl < 1) throw new Exception();
            }
            catch(Exception err)  {
                validInput = false;
                racesPerLaneTextField.setText("" + racesPerLane);
            }
            try  {
                addr = Integer.parseInt(addRacersTextField.getText());
                if(addr < 0) throw new Exception();
            }
            catch(Exception err)  {
                validInput = false;
                addRacersTextField.setText("" + addRacers);
            }

            if (validInput)  {
                numOfLanes = lanes;
                racesPerLane = rpl;
                addRacers = addr;
                calcRR = new CalculateRoundRobin();
                calculateRRThread = new Thread(calcRR);
                calculateRRThread.start();
            }
            else  {
                calculateButton.setEnabled(true);
                numberOfLanesTextField.setEnabled(true);
                racesPerLaneTextField.setEnabled(true);
                findAllRadioButton.setEnabled(true);
                findAllFilterRadioButton.setEnabled(true);
                notConsecutiveCheckBox.setEnabled(true);
                normalCalcRadioButton.setEnabled(true);
                entireLnUpRadioButton.setEnabled(true);
                addRacersTextField.setEnabled(true);
                displayAllRadioButton.setEnabled(true);
                displayRowOneRadioButton.setEnabled(true);
                displayThreeRadioButton.setEnabled(true);
                displayNoneRadioButton.setEnabled(true);
                stopButton.setEnabled(false);
                if (numOfLanes > 2)  {
                    useShortCutRadioButton.setEnabled(true);
                    entireLnUpShortRadioButton.setEnabled(true);
                }
            }
        }
    }
    private class stopListener implements ActionListener  {

        @Override
        public void actionPerformed(ActionEvent e) {
            stopButton.setBackground(Color.RED);
            calcRR.stopCalc();
        }

    }
    private class LanesFocusListener implements FocusListener {

        @Override
        public void focusGained(FocusEvent e) {
            // not used
        }

        @Override
        public void focusLost(FocusEvent e) {
            int lanes;
            try  {
                lanes = Integer.parseInt(numberOfLanesTextField.getText());
                if(lanes < 2) throw new Exception();
                if(lanes == 2)  {
                    normalCalcRadioButton.setSelected(true);
                    useShortCutRadioButton.setEnabled(false);
                    entireLnUpShortRadioButton.setEnabled(false);
                }
                else  {
                    useShortCutRadioButton.setEnabled(true);
                    entireLnUpShortRadioButton.setEnabled(true);
                }
                numOfLanes = lanes;
                idealNumOfRacersTextField.setText("" +
                        (numOfLanes * (numOfLanes -1) * racesPerLane + 1));
            }
            catch(Exception err)  {
                numberOfLanesTextField.setText("" + numOfLanes);
            }
        }
    }
    private class RacesFocusListener implements FocusListener {

        @Override
        public void focusGained(FocusEvent e) {
            // not used
        }

        @Override
        public void focusLost(FocusEvent e) {
            int rpl;
            try  {
                rpl = Integer.parseInt(racesPerLaneTextField.getText());
                if(rpl < 1) throw new Exception();
                racesPerLane = rpl;
                idealNumOfRacersTextField.setText("" +
                        (numOfLanes * (numOfLanes -1) * racesPerLane + 1));
            }
            catch(Exception err)  {
                racesPerLaneTextField.setText("" + racesPerLane);
            }
        }
    }
    private class AddRacersFocusListener implements FocusListener {

        @Override
        public void focusGained(FocusEvent e) {
            // not used
        }

        @Override
        public void focusLost(FocusEvent e) {
            int rpl;
            try  {
                rpl = Integer.parseInt(addRacersTextField.getText());
                if(rpl < 0) throw new Exception();
                addRacers = rpl;
            }
            catch(Exception err)  {
                addRacersTextField.setText("" + addRacers);
            }
        }
    }
}

